"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var experimental_utils_1 = require("@typescript-eslint/experimental-utils");
var utils_1 = require("../utils");
var node_utils_1 = require("../node-utils");
exports.RULE_NAME = 'await-async-utils';
var VALID_PARENTS = [
    'AwaitExpression',
    'ArrowFunctionExpression',
    'ReturnStatement',
];
var ASYNC_UTILS_REGEXP = new RegExp("^(" + utils_1.ASYNC_UTILS.join('|') + ")$");
function isAwaited(node) {
    return VALID_PARENTS.includes(node.type);
}
function isPromiseResolved(node) {
    var parent = node.parent;
    if (node_utils_1.isCallExpression(parent)) {
        return node_utils_1.hasThenProperty(parent.parent);
    }
    return node_utils_1.hasThenProperty(parent);
}
exports.default = experimental_utils_1.ESLintUtils.RuleCreator(utils_1.getDocsUrl)({
    name: exports.RULE_NAME,
    meta: {
        type: 'problem',
        docs: {
            description: 'Enforce async utils to be awaited properly',
            category: 'Best Practices',
            recommended: 'warn',
        },
        messages: {
            awaitAsyncUtil: 'Promise returned from `{{ name }}` must be handled',
        },
        fixable: null,
        schema: [],
    },
    defaultOptions: [],
    create: function (context) {
        var _a;
        var testingLibraryUtilUsage = [];
        return _a = {},
            _a["CallExpression > Identifier[name=" + ASYNC_UTILS_REGEXP + "]"] = function (node) {
                if (!isAwaited(node.parent.parent) && !isPromiseResolved(node)) {
                    testingLibraryUtilUsage.push(node);
                }
            },
            _a['Program:exit'] = function () {
                testingLibraryUtilUsage.forEach(function (node) {
                    var variableDeclaratorParent = node.parent.parent;
                    var references = (variableDeclaratorParent.type === 'VariableDeclarator' &&
                        context
                            .getDeclaredVariables(variableDeclaratorParent)[0]
                            .references.slice(1)) ||
                        [];
                    if (references &&
                        references.length === 0 &&
                        !isAwaited(node.parent.parent) &&
                        !isPromiseResolved(node)) {
                        context.report({
                            node: node,
                            messageId: 'awaitAsyncUtil',
                            data: {
                                name: node.name,
                            },
                        });
                    }
                    else {
                        for (var _i = 0, references_1 = references; _i < references_1.length; _i++) {
                            var reference = references_1[_i];
                            var referenceNode = reference.identifier;
                            if (!isAwaited(referenceNode.parent) &&
                                !isPromiseResolved(referenceNode)) {
                                context.report({
                                    node: node,
                                    messageId: 'awaitAsyncUtil',
                                    data: {
                                        name: node.name,
                                    },
                                });
                                break;
                            }
                        }
                    }
                });
            },
            _a;
    },
});
